<?php

namespace EventPortal\Domain\Agenda\Actions;

use EventPortal\Domain\Agenda\Models\Session;
use EventPortal\Domain\Agenda\SessionInput;
use EventPortal\Domain\Agenda\SessionOverlap;
use EventPortal\Domain\Form\Facades\Form;

class UpdateSession
{
    public function execute(Session $session, array $input = []): Session
    {
        $input = SessionInput::from($input)
            ->forceDate($session->pathway->agenda->date)
            ->formatForDatabase();

        $form = Form::make('session.update')
            ->model($session)
            ->useAlternativeNames()
            ->validate($input);

        $overlaps = Session::query()
            ->where('id', '!=', $session->id)
            ->completelyOverlaps($input['starting_at'], $input['ending_at'])
            ->exists();

        SessionOverlap::throwValidationExceptionIfOverlapping($form->getValidator(), $overlaps);

        return $form->save($input)->getModel();
    }
}
