<?php

namespace EventPortal\Domain\Util\Sidebar;

use Illuminate\Contracts\Auth\Authenticatable;
use Illuminate\Database\Eloquent\Model;

class Sidebar
{
    /** @var \Illuminate\Support\Collection */
    protected $actions = [];

    /** @var string|null */
    protected $title;

    /** @var Model */
    protected $model;

    /** @var Authenticatable */
    protected $user;

    public function title(string $title): self
    {
        $this->title = $title;

        return $this;
    }

    public function user(Authenticatable $user): self
    {
        $this->user = $user;

        return $this;
    }

    public function model(Model $model): self
    {
        $this->model = $model;

        return $this;
    }

    public function addLinks(callable $callable): self
    {
        $this->actions[] = $callable;

        return $this;
    }

    public function render(): array
    {
        $this->resolveLinks()
            ->filterPermittedLinks()
            ->renderLinks();

        return [
            'title' => $this->title,
            'children' => $this->actions->toArray(),
        ];
    }

    private function resolveLinks(): self
    {
        $this->actions = collect($this->actions)
            ->transform(function ($action) {
                return $action($this->model);
            })
            ->flatten(1); // addLinks can be called multiple times

        return $this;
    }

    private function filterPermittedLinks(): self
    {
        $this->actions = $this->actions->filter(function (Link $link) {
            return $this->hasPermission($link->permission);
        });

        return $this;
    }

    private function renderLinks(): self
    {
        $this->actions = $this->actions->transform(function (Link $link) {
            return $link->render();
        });

        return $this;
    }

    private function hasPermission(?string $permission): bool
    {
        return $permission === null || optional($this->user)->hasPermissionTo($permission) === true;
    }
}
