<?php

namespace EventPortal\Http\Controllers\Admin\Delegate;

use EventPortal\Domain\Delegate\Models\Delegate;
use EventPortal\Domain\Form\Facades\Form;
use EventPortal\Domain\Table\Facades\Table;
use EventPortal\Domain\User\Models\User;
use EventPortal\Domain\Util\Breadcrumb;
use EventPortal\Domain\Util\Render;
use EventPortal\Http\Controllers\Controller;
use Illuminate\Http\RedirectResponse;
use Illuminate\Http\Request;
use Inertia\Response;

class DelegateController extends Controller
{
    public function __construct()
    {
        $this->middleware('check-permission:delegate.show')->only('show');
        $this->middleware('check-permission:delegate.store')->only(['create', 'store']);
        $this->middleware('check-permission:delegate.update')->only(['edit', 'update']);
        $this->middleware('check-permission:delegate.destroy')->only('delete');
    }

    public function index(): Response
    {
        $table = Table::make('delegate.index')
            ->query(Delegate::joinRelationship('user'))
            ->request(request())
            ->get();

        $breadcrumbs = Breadcrumb::make()
            ->add('Delegates', route('admin.delegate.index'))
            ->get();

        return Render::component('Delegate/Index')
            ->key('delegate.index')
            ->withTable($table)
            ->withBreadcrumbs($breadcrumbs)
            ->get();
    }

    public function create(): Response
    {
        $form = Form::make('delegate.create')
            ->model(new User)
            ->render();

        $breadcrumbs = Breadcrumb::make()
            ->add('Delegates', route('admin.delegate.index'))
            ->add('Create a delegate')
            ->get();

        return Render::component('Delegate/Create')
            ->key('delegate.create')
            ->withForm($form)
            ->withBreadcrumbs($breadcrumbs)
            ->get();
    }

    public function store(Request $request): RedirectResponse
    {
        Form::make('delegate.create')
            ->model(new User)
            ->validate($request->all())
            ->save($request->all());

        return redirect()->route('admin.delegate.index');
    }

    public function edit(Delegate $delegate): Response
    {
        $form = Form::make('delegate.edit')->model($delegate)->render();

        $breadcrumbs = Breadcrumb::make()
            ->add('Delegates', route('admin.delegate.index'))
            ->add($delegate->name, route('admin.delegate.show', $delegate))
            ->add('Edit delegate')
            ->get();

        return Render::component('Delegate/Edit')
            ->key('delegate.edit')
            ->withForm($form)
            ->withBreadcrumbs($breadcrumbs)
            ->withTitle($delegate->name)
            ->get();
    }

    public function update(Request $request, Delegate $delegate): RedirectResponse
    {
        Form::make('delegate.edit')
            ->model($delegate)
            ->validate($request->all())
            ->save($request->all());

        return redirect()->route('admin.delegate.index');
    }

    public function show(Delegate $delegate): Response
    {
        $form = Form::make('delegate.show')->model($delegate)->render();

        $breadcrumbs = Breadcrumb::make()
            ->add('Delegates', route('admin.delegate.index'))
            ->add($delegate->name)
            ->get();

        return Render::component('Delegate/Show')
            ->key('delegate.show')
            ->withForm($form)
            ->withBreadcrumbs($breadcrumbs)
            ->withTitle($delegate->name)
            ->get();
    }
}
