<?php

namespace EventPortal\Seeds;

use EventPortal\Domain\Permission\Models\Group;
use EventPortal\Domain\Permission\Models\Permission;
use Illuminate\Database\Seeder;
use Illuminate\Support\Str;

class PermissionSeeder extends Seeder
{
    public function run(): void
    {
        $permissions = collect([
            'Delegate Management' => [
                [
                    'name' => 'delegate.show',
                    'label' => 'View Delegates',
                ],
                [
                    'name' => 'delegate.store',
                    'label' => 'Create Delegates',
                ],
                [
                    'name' => 'delegate.update',
                    'label' => 'Edit Delegates',
                ],
                [
                    'name' => 'delegate.destroy',
                    'label' => 'Delete Delegates',
                ],
            ],
            'Event Management' => [
                [
                    'name' => 'event.show',
                    'label' => 'View Events',
                ],
                [
                    'name' => 'event.store',
                    'label' => 'Create Events',
                ],
                [
                    'name' => 'event.update',
                    'label' => 'Edit Events',
                ],
                [
                    'name' => 'event.destroy',
                    'label' => 'Delete Events',
                ],
            ],
            'Team Management' => [
                [
                    'name' => 'team.show',
                    'label' => 'View Teams',
                ],
                [
                    'name' => 'team.store',
                    'label' => 'Create Teams',
                ],
                [
                    'name' => 'team.update',
                    'label' => 'Edit Teams',
                ],
                [
                    'name' => 'team.destroy',
                    'label' => 'Delete Teams',
                ],
            ],
            'User Management' => [
                [
                    'name' => 'user.show',
                    'label' => 'View Users',
                ],
                [
                    'name' => 'user.store',
                    'label' => 'Create Users',
                ],
                [
                    'name' => 'user.update',
                    'label' => 'Edit Users',
                ],
                [
                    'name' => 'user.destroy',
                    'label' => 'Delete Users',
                ],
            ],
            'Delegate Management' => [
                [
                    'name' => 'delegate.show',
                    'label' => 'View Delegates',
                ],
                [
                    'name' => 'delegate.store',
                    'label' => 'Create Delegates',
                ],
                [
                    'name' => 'delegate.update',
                    'label' => 'Edit Delegates',
                ],
                [
                    'name' => 'delegate.destroy',
                    'label' => 'Delete Delegates',
                ],
            ],
        ]);

        $permissions->each(static function (array $permission, string $group): void {
            $group = Group::firstOrCreate([
                'name' => Str::slug($group),
                'label' => Str::ucfirst($group),
            ]);

            collect($permission)->each(static function (array $permission) use ($group): void {
                Permission::updateOrCreate([
                    'name' => $permission['name'],
                    'label' => $permission['label'],
                    'guard_name' => 'web',
                    'group_id' => $group->id,
                ]);
            });
        });
    }
}
